;****************************************************************************
;*			File name: eva.asm												*
;*			Date created: 30th June 2009									*
;*			Author: Tan Ghee Siak											*
;****************************************************************************

;Random ASCII character generator
;generates numbers 0-9 and letters a-z (lower case)

;Call With: Nothing

;  Returns: eax = al = pseudo-random ASCII character
Randm		proc
        INVOKE  Sleep,0
        sub     esp,8
        INVOKE  QueryPerformanceCounter,esp
        pop     eax
        xor     eax,[esp]
        add     esp,4
        xor     edx,edx
        mov     ecx,36
        div     ecx
        xchg    eax,edx
        add     eax,30h
        cmp     eax,39h
        jbe     Randm0

        add     eax,27h

Randm0: 
		ret

Randm   ENDP

nseed		proc 	TheSeed:DWORD
    		mov 	eax, TheSeed
    		mov 	nrandom_seed, eax

    		ret
nseed 		endp

nrandom 	PROC base:DWORD

    		mov 	eax, nrandom_seed

  			; ****************************************
    		test 	eax, 80000000h
    		jz  	@F
    		add 	eax, 7fffffffh
  			@@:   
		  	; **************************************** 

    		xor 	edx, edx
    		mov 	ecx, 127773
    		div 	ecx
    		mov 	ecx, eax
    		mov 	eax, 16807
    		mul 	edx
    		mov 	edx, ecx
    		mov 	ecx, eax
    		mov 	eax, 2836
    		mul 	edx
    		sub 	ecx, eax
    		xor 	edx, edx
    		mov 	eax, ecx
    		mov 	nrandom_seed, ecx
    		div 	base

    		mov 	eax, edx
    		ret

nrandom ENDP

; returns an integer between 0 and Range instead of 0 and TRAND_MAX
TRand 		proc 	Range:DWORD   
    		LOCAL 	TempInt:DWORD
    		LOCAL 	RMax:DWORD

;    		mov 	eax, TRAND_MAX;
			mov		eax,Range
    		mov 	RMax, eax;
      
    		mov  	eax, Seed;
    		imul 	eax, eax,343FDh;
    		add  	eax, 269EC3h;
		    mov  	Seed,eax;
    		sar  	eax,10h;
    		and  	eax,7FFFh;

    		mov 	TempInt, eax;

    		fild 	TempInt;
    		fild 	RMax;
    		fdivp 	st(1), st(0);
    		fild 	Range;
    		fmulp 	st(1), st(0);
    		fistp 	TempInt; 
 
    		mov 	eax, TempInt;
    		ret
    
TRand 		endp

;*==========================================================================*
;*			Routine to initialise for random numbers						*
;*--------------------------------------------------------------------------*
;* 			syntax :														*
;* 			call InitRndmNbr												*
;*																			*
;* 			Return:															*
;* 			eax = number of ticks since the computer started, 				*
;*				  and Seed defined											*
;*==========================================================================*
InitRndmNbr	PROC
			push 	ecx
			push 	edx

			invoke	GetTickCount
			test 	eax,eax 							; ) to be sure it's not zero (it's possible ! if you launch the program 48 days after you've started the computer, lol)
			jnz 	Label1 								; ) (and depending of the radom number routine, some really don't like zeros...)
			mov 	eax,123456789 ; )
	Label1: 
			mov 	Seed,eax 								; save the seed

			pop 	edx
			pop 	ecx
			ret
InitRndmNbr ENDP

;*==========================================================================*
;* 				Routine to generate a random number							*
;*--------------------------------------------------------------------------*
;* 				syntax :													*
;* 				mov ebx,Range												*
;* 				call GetRndmNbr												*
;*																			*
;* 			Return:															*
;* 				eax = desired random number									*
;*==========================================================================*
GetRndmNbr PROC
			push 	edx

			mov 	eax,987654321 		; a x value
			mul 	Seed 				; multiply eax by your Seed
			bswap 	eax 				; swap bytes (to not have a too easy suit)
			xor 	eax,078787878h 		; apply a mask
			mov 	Seed,eax 			; save the new RndmNbr as Seed, for next use
			; a range ?
			inc 	ebx
			jz 		Label2 				; if not jump Label2
			; here a range :
			mul 	ebx 				; mul by range+1
			mov 	eax,edx 			; return the result in eax
			; exit
Label2: 
			dec 	ebx ; restore ebx
			
			pop 	edx
			ret
GetRndmNbr ENDP

rand_range	proc
			mov		eax,-1				;
			.while (eax > ebx) 
				call	GetRndmNbr
			.endw
			
			.if eax < 0
				mov		eax,0
			.elseif eax > ebx
				mov		eax,ebx
			.endif
			ret
rand_range	endp

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Random number generator
;;
;;      Unlike in RAND we cannot keep the old random value in EBP
;;      because we are called by the Windows timer. Therefore, we use
;;      a global variable called seed.
;;
;;      Returns new random number in EAX.
;;      Sets EDX = 0.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
align   DWORD
random  proc    private

        mov     eax, 214013t
        imul    Seed
        sub     edx, edx                ; Prevent divide overflow by caller
        add     eax, 2531011t
        mov     Seed, eax
        ret

random  endp

;*==========================================================================*
;*			Routine to check byte bit state									*
;*--------------------------------------------------------------------------*
;*			Input:	AL = Reference data 									*
;*					AH = Data for check										*
;*																			*
;*			Output:	AL = Number of errors 									*
;*																			*
;*			Variables used													*
;*			NoBitErr - number of errors found								*
;*			BitErrList - list contain the bit number in error				*
;*==========================================================================*
check_byte_err	proc
				push	edx						; save EDX
				push	ecx						;  & ECX
				push	ebx						;  & EBX
;				push	eax						;  & EAX
				
				;*====	Setup the environment	====
				mov		[NoBitErr],0			; reset count
				mov		ecx,08					; check all bit 8
				sub		ebx,ebx					; start bit offset
				sub		edx,edx					; number of errors
				
				;*====	Check loop	====
				.while ecx > 0					; check all the 8-bits
					mov		bx,ax				; copy into BX
					and		bx,0101h			; check the last bit
					xor		bl,bh				; check if the bits are the same
					jz		samebit				; Same, go to next bit
					
					;*====	Bit incorrect	====
					mov		[BitErrList+edx],bl	; write bit number
					inc		[NoBitErr]			; increment error count
					inc		edx					; increment error count
					
					;*====	Prepare for the next bit	====
		samebit:
					shl		al,1				; ignore processed bit
					shl		ah,1				; 
					
					inc		ebx					; increment bit number
					dec		ecx					; decrement bit counter
				.endw
				
				;*====	Restore registers	====
				mov		al,dl					; AL = number of error
;				pop		eax
				pop		ebx
				pop		ecx
				pop		edx
				ret
check_byte_err	endp

;*==========================================================================*
;*			Routine to check same bit current state							*
;*--------------------------------------------------------------------------*
;*			Input:	AL = Reference data 									*
;*					AH = Data for check										*
;*				  	CL = Bit position, zero based							*
;*																			*
;*			Output: BL = Same bit state										*
;*						 00 = Same, 1 = Different							*
;*==========================================================================*
chk_samebit		proc
				push	dx						;
				push	ax
				
				mov		dl,1					; 
				shl		dl,cl					; move to bit position
				mov		dh,dl					; 
				and		ax,dx					; get bit state
				xor		al,ah					; check state difference - same = 0, different > 0
				mov		bl,al					; return check state
				
				pop		ax
				pop		dx
				ret
chk_samebit		endp

;*==========================================================================*
;*			Routine to extract bit current state							*
;*--------------------------------------------------------------------------*
;*			Input:	AL = Data information - 8bit							*
;*				  	CL = Bit position, zero based							*
;*			Output: AH = Updated data state									*
;*						 00 = reset, > 0 = Set								*
;*==========================================================================*
get_bitstte		proc
				shr		al,cl					; move to bit position
				and		al,01					; 
				ret
get_bitstte		endp

;*==========================================================================*
;*			Routine to extract the current bit state						*
;*--------------------------------------------------------------------------*
;*			Input:	AL = Data information									*
;*				  	CL = Bit position, zero based							*
;*			Output: BL = Bit state											*
;*==========================================================================*
get_bitstate	proc
				cmp		cl,00					; at bit 0
				je		getbst					; Yes, then get bit state
				
				;*====	Bit position 0<x<8	====
				dec		cl						; decrement all others
				shr		al,cl					; shift data
		getbst:
				and		al,01					; mask out all others
				ret
get_bitstate	endp

;*==========================================================================*
;*			Routine to set the bit to new state								*
;*--------------------------------------------------------------------------*
;*			Input:	AL = Data information									*
;*				  	CL = Bit position, zero based							*
;*			Output: AL = Updated data state									*
;*==========================================================================*
set_bitstate	proc
				mov		ah,1					; set bit mask 
				shl		ah,cl					; move to bit position
				or		al,ah					; set the required bit
				ret
set_bitstate	endp

;*==========================================================================*
;*			Routine to reset the bit to new state							*
;*--------------------------------------------------------------------------*
;*			Input:	AL = Data information									*
;*				  	CL = Bit position, zero based							*
;*			Output: AL = Updated data state									*
;*==========================================================================*
rset_bitstate	proc
				mov		ah,1					; set bit mask 
				shl		ah,cl					; move to bit position
				not		ah						; compliment the bit
				and		al,ah					; set the required bit
				ret
rset_bitstate	endp

;*==========================================================================*
;*			Routine to toggle bit and return updated string					*
;*--------------------------------------------------------------------------*
;*			Input:	AL = Data information									*
;*				  	CL = Bit position, zero based							*
;*			Output: AL = Updated data state									*
;*					DL = toggle inverted state (only one bit is set if set	*
;*==========================================================================*
toggle_bit		proc
				mov		ch,al							; copy AL
				mov		ah,1							; set AH = 1 
				shl		ah,cl							; move to bit position
				and		al,ah							; AND the data to get current state
				not		al								; invert current bit information
				and		al,ah							; ignore other bits
				mov		dl,al							; copy inverted bit state
				xor		ch,ah							; 
				or		al,ch							; place the state of the other bits
				ret	
toggle_bit		endp

;*==========================================================================*
;*			Routine to toggle bit or Port/Tris and return updated string	*
;*--------------------------------------------------------------------------*
;*			Input:	AL = Data information									*
;*				  	CL = Bit position, zero based							*
;*					DH = 0: Port, 1: TRIS									*
;*			Output: AL = Updated data state									*
;*==========================================================================*
toggle_PTRIS	proc
				;push	edx								;
				call	toggle_bit						; toggle the required bit
				
				;pop		edx
				cmp		dh,00							; Port data?
				jne		togTRIS							; No, then must be TRIS data
				
				mov		esi,offset szBit0				; default to zero bit
				cmp		dl,00							; current bit = 0?
				je		togdone							; Yes, then exit
				mov		esi,offset szBit1				; data = 1
				jmp		togdone							;
				
				;*====	Process for TRIS data	====
		togTRIS:
				mov		esi,offset szOut				; default to OUT
				cmp		dl,00							;
				je		togdone							; Yes, then exit
				mov		esi,offset szIn					; input mode
		togdone:
				ret
toggle_PTRIS	endp

;*==========================================================================*
;*			Routine to toggle port bit and return updated string			*
;*--------------------------------------------------------------------------*
;*			Input:	AL = Data information									*
;*				  	CL = Bit position, zero based							*
;*			Output: AL = Updated data state									*
;*					ESI -> Label string										*
;*==========================================================================*
toggle_portbit	proc
				mov		ah,al							; copy into AH
				mov		ch,cl							;  & CL into CH
				call	get_bitstate					; BL = bit state
				mov		al,ah							; copy into AL
				mov		cl,ch							; restore CL bit number
	
				;*====	Check current bit state	====			
				.if al == 0								; currently in reset state
					;*====	In reset state, set it	====
					mov		esi,offset szBit1			; toggle it to 1
					call	set_bitstate				; set the bit to
				.else
					;*====	Currently in set state	====
					;*====	In set state, reset it	====
					mov		esi,offset szBit0			; toggle it to zero
					call	rset_bitstate				; reset the bit 
				.endif
				ret
toggle_portbit	endp

;*==========================================================================*
;*			Routine to toggle port bit and return updated string			*
;*--------------------------------------------------------------------------*
;*			Input:	AL = Data information									*
;*				  	CL = Bit position, zero based							*
;*			Output: AL = Updated data state									*
;*					ESI -> Label string										*
;*==========================================================================*
toggle_trisbit	proc
				mov		ah,al							; copy into AH
				mov		ch,cl							;  & CL into CH
				call	get_bitstate					; BL = bit state
				mov		al,ah							; copy into AL
				mov		cl,ch							; restore CL bit number
	
				;*====	Check current bit state	====			
				.if al == 0								; currently in reset state
					;*====	In output state, toggle to input 	====
					mov		esi,offset szIn			; toggle it to 1
					call	set_bitstate				; set the bit to
				.else
					;*====	Currently in input state	====
					;*====	Change it to output state	====
					mov		esi,offset szOut			; toggle it to output
					call	rset_bitstate				; reset the bit 
				.endif
				ret
toggle_trisbit	endp

;*==========================================================================*
;*			Routine to check byte for bit inconsistencies					*
;*--------------------------------------------------------------------------*
;*			Input:	AL = Ref data information								*
;*					AH = Data to check										*
;*			Output: BitErrList contains bit position of error found in		*
;*					NoBitErr contains the number of bit error				*
;*==========================================================================*
ChkByteBit		proc
				push	ebx								; save EBX
				push	ecx								;
				push	edx								;
				
				mov		ecx,8							; 8 bits to check
				sub		edx,edx							; reset EDX
				mov		esi,offset BitErrList			;
				mov		[NoBitErr],0					; reset count
				
				.while ecx > 0
					mov		bx,ax						; make a copy of the information
					and		bx,0101h					; mask out other bits
					
					cmp		bh,bl						; check if bit information are the same
					je		chkbytnxt					; Yes, go to next bit
					
					;*====	Save bit that is different	====
					mov		[esi],dl					; save bit position
					inc		[NoBitErr]					;  & increment count
					inc		esi							; increment ptr
		chkbytnxt:
					inc		edx							; increment bit position number
					dec		ecx							; decrement loop count
				.endw
				
				pop		edx
				pop		ecx								;
				pop		ebx								;
				ret
ChkByteBit		endp

;*==========================================================================*
;*			Routine to randomize instruction settings						*
;*--------------------------------------------------------------------------*
;*			Game of MAX_INSTRUCTIONS instructions							*
;*			Set of 25 instructions											*
;*			Routine assumes the random seed has been initialized			*
;*==========================================================================*
randomize		proc
				mov		ecx,MAX_INSTRUCTIONS			; maximum instruction count
				sub		esi,esi							; reset offset

				.while ecx > 0
					push	ebx							; save offset
					push	ecx							;  & count
					
					;*====	Randomize for instruction	====
					mov		ebx,3						; EBX = maximum range
					;call	GetRndmNbr					; get random value
					call	rand_range					; get range call
					mov		[InstrList+esi],al			; save random number
					
					;*====	Randomize for Port number	====
					mov		ebx,3						; EBX = maximum range
;					call	GetRndmNbr					; get random value
					call	rand_range					; get range call
					mov		[IPortList+esi],al			; 
					
					;*====	Randomize for Port bit	====
					mov		ebx,7						; EBX = maximum range
;					call	GetRndmNbr					; get random value
					call	rand_range					; get range call
					mov		[IBitList+esi],al			; 
					
					
					;*====	Randomize for Port data	====
					mov		ebx,255						; EBX = maximum range
;					call	GetRndmNbr					; get random value
					call	rand_range					; get range call
					mov		[IDataList+esi],al			; 
					
					;*====	Randomize for data type	====
;					mov		esi,offset IDTypList		; ESI -> data type list
					mov		ebx,1						; EBX = maximum range
;					call	GetRndmNbr					; get random value
					call	rand_range					; get range call
					mov		[IDTypList+esi],al			; 
					
;					invoke	GetTickCount
;					mov		[Seed],eax
					
					;*====	Randomize for the next instruction	====
					pop		ecx							; restore ECX
					pop		ebx							;  & EBX
					
					inc		esi							; increment offset
					dec		ecx							; decrement count
				.endw
				ret
randomize		endp

;*==========================================================================*
;*			Routine to load instruction information onto memory				*
;*--------------------------------------------------------------------------*
;*			Input:	EAX = Line number in instruction list					*
;*			Output variables used:											*
;*			CurInstr, CurPort, CurBit, CurData, CurDTyp						*
;*==========================================================================*
ld_instr		proc
ld_instr		endp

;*==========================================================================*
;*			Routine to format instruction into string for display			*
;*--------------------------------------------------------------------------*
;*			Input variables used:											*
;*			CurInstr, CurPort, CurBit, CurData, CurDTyp						*
;*==========================================================================*
fmt_instr		proc
				mov		esi,offset TRIS_MSG				; default to TRIS message
				mov		ah,[CurInstr]					; AH = current instruction
				cmp		ah,0							; TRIS instruction?
				je		fmtPort							; Yes,continue
				
				mov		esi,offset TRISbits_MSG			; default to TRISbit message
				cmp		ah,1							; TRIS instruction?
				je		fmtPort							; Yes,continue
				
				mov		esi,offset PORT_MSG				; default to PORT message
				cmp		ah,2							; TRIS instruction?
				je		fmtPort							; Yes,continue
				
				mov		esi,offset PORTbits_MSG			; default to PORTbit message
				cmp		ah,3							; TRIS instruction?
				je		fmtPort							; Yes,continue
				
				;*====	Error, incorrect random value	=====
		fmterr:
				cmp		ah,4							; > 4?
				jb		fmterr1							;
				
				mov		ah,3							; set to max of 3
				mov		[CurInstr],3
				jmp		fmtPort
				
		fmterr1:
				cmp		ah,0							; > 4?
				jae		fmtPort							;
				mov		[CurInstr],0					; set to value of 0
				mov		ah,0
				
				;*====	Format port information	====
		fmtPort:
				mov		ebx,04							; offset to character
				
				mov		al,[CurPort]					; get port number
				
				;*====	Format into instruction string	====
				add		al,41h							; ASCII encode 
				mov		[esi+ebx],al					; write port number into instruction

				.if ah == 1	|| ah == 3					; TRISbit or PORTbit instruction
					;*====	TRISbit and PORTbit instruction	====
					
					.if ah == 1
						add		ebx,10					; skip 'TRIS number bits.
					.elseif ah == 3						; PORTbit instruction
						add		ebx,7					; skip 'PORT number bits.
					.endif
					
					mov		[esi+ebx],al				; put the port name
					mov		ah,[CurBit]					; AH = bit number
					inc		ebx							; skip byte
					
					add		ah,30h						; encode ASCII code
					mov		[esi+ebx],ah				; save bit number
					
					;*====	Skip the equate character	====
					add		ebx,4						; skip the equate character
					
					mov		cl,[CurBit]					; CL = bit number
					mov		al,[CurData]				; AL = port data
					shr		al,cl						; move to bit position
					and		al,01						; Al = bit state
					add		al,30h
					mov		[esi+ebx],al				; save the bit value
				.else
					;*====	Skip the equate character	====
					add		ebx,5							; skip the equate character
				
					;*====	Format the data for display: data type	=====
					sub		eax,eax							; reset EAX
					mov		al,[CurData]					; AL = port data
				
					.if [CurDTyp] == 0
						;*====	Binary data type	=====
						mov		byte ptr [esi+ebx],'b'		; indicate with binary data type
						mov		ecx,08						; encode 8 bytes of binary data
						.while ecx > 0
							inc		ebx						; skip last byte
						
							mov		byte ptr [esi+ebx],'0'	; default to '0'
							shl		al,1					; shift left
							jnc		donxtbit				; bit zero so skip next instruction
						
							mov		byte ptr [esi+ebx],'1'	; write '1'
			donxtbit:
							dec		ecx						; decrement count
						.endw
					.else
						;*====	Hexadecimal data type	=====
						mov		byte ptr [esi+ebx],'x'		; indicate with binary data type
						inc		ebx							; increment offset
					
						mov		ah,al						; copy data into AH
						shr		al,4						; shift into lower nybble
						and		ax,0F0Fh					; last out MS nybble
	
						;*====	Check upper nybble for hexadecimal value	====				
						cmp		ah,10						; check for hexadecimal value
						jb		donxtnyb					; continue with next nybble
					
						add		ah,7						; set to hexadecimal value

			donxtnyb:
						;*====	Check lower nybble for hexadecimal value	====				
						cmp		al,10						; check for hexadecimal value
						jb		dobyte						; continue with next nybble
					
						add		al,7						; set to hexadecimal value
			dobyte:
						add		ax,3030h					;
						mov		[esi+ebx],ax				; save upper byte
						inc		ebx
;						mov		[esi+ebx],al				; save lower byte
					.endif
				.endif
				
				inc		ebx								; skip last entered data
				mov		byte ptr [esi+ebx],' '			;
				inc		ebx
				mov		byte ptr [esi+ebx],';'			;
				inc		ebx
				mov		byte ptr [esi+ebx],0			; terminate string
				
				ret
fmt_instr		endp


;*==========================================================================*
;*			Routine to display on page of instruction 						*
;*==========================================================================*
dsp_instr_page	proc	hWin:DWORD
				push	ebx
				push	ecx
				push	edx
				
				;*====	Setup for the display of one page	====
				mov		ecx,25							; 25 instructions
				sub		edx,edx							; reset EDX
				mov		ebx,edx							;
				mov		al,[CurSet]						; get current set
				mov		bl,25							;
				mul		bl								; AL = total
				mov		bl,al							; LB = start offset to instruction
				mov		[PortA_ID],800					; start ID
				mov		edx,800
				
				;*====	Prepare to displat one page of instruction	====
				.while ecx > 0
					push	ecx							; save ECX
					push	ebx							;  & BEX
					push	edx
					
					;*====	Load up instruction information	====
					mov		al,byte ptr [InstrList+ebx]	; AL = instruction
					mov		[CurInstr],al				; save it
					
					mov		al,byte ptr [IPortList+ebx]	; AL = port name
					mov		[CurPort],al				; save it
					
					mov		al,byte ptr [IBitList+ebx]	; AL = bit number
					mov		[CurBit],al					; save it
					
					mov		al,byte ptr [IDataList+ebx]	; AL = data
					mov		[CurData],al				; save it
					
					mov		al,byte ptr [IDTypList+ebx]	; AL = data type
					mov		[CurDTyp],al				; save it
					
					call	fmt_instr					; format the instruction
					
					;*====	Display the instruction string	====
					invoke	SetDlgItemText,hWin, edx, esi

					pop		edx					
					pop		ebx					
					pop		ecx							; restore ECX
					inc		edx
					inc		ebx							; increment offset
					dec		ecx
				.endw
				
				pop		ebx
				pop		edx
				pop		ecx
				ret
dsp_instr_page	endp

;*				Input:	ESI -> Buffer	
;*						AL = Hex data
disp_hex		proc
				mov		ah,al							;
				shr		al,4
				and		ax,0f0fh
				cmp		ah,10
				jl		dnumbh
				add		ah,07
		dnumbh:
		
				cmp		al,10
				jl		dnumbl
				add		al,07
		dnumbl:
				add		ax,3030h
				mov		[esi],ax
				add		esi,2
				ret
disp_hex		endp

;*				Input:	ESI -> Buffer	
;*						AX = Hex data
disp_whex		proc
				mov		ebx,eax
				mov		al,ah
				call	disp_hex
				mov		al,bl
				call	disp_hex
				ret
disp_whex		endp

;*				Input:	ESI -> Buffer	
;*						EAX = Hex data
disp_dwhex		proc
				push	eax
				shld	ebx,eax,16				;
				mov		ax,bx
				call	disp_whex
				pop		eax
				call	disp_whex
				ret
disp_dwhex		endp

;*==========================================================================*
;*			Routine to convert binary byte into ASCII code					*
;*--------------------------------------------------------------------------*
;*			This routine will display 3 digits of ASCII value				*
;*			Input:	AL = Binary value										*
;*					ESI -> String buffer									*
;*			Output:	Data updated in string null terminated					*
;*==========================================================================*
cnv_b_ascii		proc
				push	bx						; save BX
				
				sub		edx,edx					; reset EDX
				mov		bh,10					; 
				mov		ah,20h					;
				mov		[esi],ah				;
				mov		[esi+1],ah				;
				mov		[esi+2],ah				;
				mov		byte ptr [esi+3],0		; set null character
				add		esi,3					; 
				sub		ah,ah
		cnvblp:
				div		bh						; divide by 10
				add		ah,30h					; encode ASCII
				mov		[esi],ah				; save digit
				dec		esi						; decrement ESI
				
				cmp		al,9					; > 9?
				ja		cnvblp					; No, then exit from conversion
				
				;*====	Exit from conversion	====
				add		al,30h					; encode ASCII
				mov		[esi],al				; save digit
				
				pop		bx
				ret
cnv_b_ascii		endp

;*==========================================================================*
;*			Routine to setup the entered data with that in the 				*
;*			required data										 			*
;*--------------------------------------------------------------------------*
;*			Input variables used:											*
;*			CurInstr, CurPort, CurBit, CurData, CurDTyp, PData				*
;*==========================================================================*
check_data		proc	hWin:DWORD
				mov		al,[CurPort]			; get current port number
				mov		ah,[CurInstr]			; AH = instruction type
				.if al == 0
					.if ah == 1 && ah == 3
						mov		ah,[PortA_D]	; Port A data
					.else
						mov		ah,[TrisA_D]	; Tris A data
					.endif
				.elseif al == 1
					.if ah == 1 && ah == 3
						mov		ah,[PortB_D]
					.else
						mov		ah,[TrisB_D]	; Tris A data
					.endif
				.elseif al == 2
					.if ah == 1 && ah == 3
						mov		ah,[PortC_D]
					.else
						mov		ah,[TrisC_D]	; Tris A data
					.endif
				.elseif al == 3
					.if ah == 1 && ah == 3
						mov		ah,[PortD_D]
					.else
						mov		ah,[TrisD_D]	; Tris A data
					.endif
				.endif
				
				mov		[PData],ah				; save it into PData for processing
				mov		al,[CurData]			; AL = given port data
				
				;*====	Check the instruction type	====
				.if (CurInstr == 1) || (CurInstr == 3)
					;*====	Bit definition	====
					mov		cl,[CurBit]			; CL = bit number
					call	chk_samebit			; check if data is the same
					.if bl > 0
						mov		[BitErrList],cl	; write bit number in list
						mov		[NoBitErr],1	; indicate 1 error
						mov		al,1			; 1 error
					.else
						sub		al,al			; return OK status
					.endif
				.else
					;*====	Byte definition	====
					.if al == ah
						mov		al,0			; return no error
					.else
						;*====	Different between port data and entered data	====
						call	check_byte_err		; check for byte error
						.if [NoBitErr] == 0
							mov		al,[NoBitErr]	; AL = number of bit error
						.endif
					.endif
				.endif

				;*====	Process correct or incorrect data	====				
				.if al > 0					; bit is different
						
					sub		[CurEnergy],al		; subtract error count from energy level
	        		mov		esi,offset szEngyLoss ; ESI -> string buffer
    	    		call	cnv_b_ascii			; convert binary to ASCII
					
					mov		al,[CurEnergy]		;
        			mov		esi,offset szEngyLeft ; ESI -> string buffer
        			call	cnv_b_ascii			; convert binary to ASCII
					invoke	MessageBox,hWin,ADDR szEngyLoss,ADDR szDisplayName,MB_OK
					
					;*====	Indicate which bit or bits were wrong	====
					
					.if [CurEnergy] == 0	; no more energy left
       		restart:
						invoke MessageBox,hWin,ADDR szNoEnergy,ADDR szDisplayName,MB_OKCANCEL
              				
  						.if eax == IDOK		; Yes, restart with new energy level
              				;*====	Restart with a new game and update all variables	====
              			.else
              				;*====	Quit from the game	====
              				invoke MessageBox,hWin,ADDR szQuit,ADDR szDisplayName,MB_OKCANCEL
              				.if eax == IDOK
              				.else
              					jmp		restart	; ask again
              				.endif
              			.endif
					.endif
				.else						; bit is the same
					;*====	Same bit found. Proceed to next instruction	====
					inc		[CurLine]		; increment run line
					invoke	SetDlgItemText,hWin,902,offset szClick	; change button to click instruction
		        	invoke  SendMessage,hStatus,SB_SETTEXT,0, offset szClickInstr	; display on status 
				.endif
				ret
check_data		endp

;*==========================================================================*
;*			Routine to increment second and display current time elapsed	*
;*--------------------------------------------------------------------------*
;*			Input variables used:											*
;*			MinStr,	Secstr													*
;*==========================================================================*
show_etime		proc
				mov		esi,offset SecStr 	;
				mov		ax,[esi]			; increment the LS digit
				inc		ah					; 
				cmp		ah,3Ah				; > 39?
				jl		doSecMS				; no, then continue
				
				;*====	Increment MS digit	====
				mov		ah,30h				; reset to  0
				inc		al					; increment MS sec digit
				
				;*====	Save the data	====
		doSecMS:
				mov		[esi],ax			;
				
				cmp		al,3Ah				; > 9?
				jl		doshowex			; No, then exit
				
				mov		esi,offset MinStr 	;
				
				mov		ax,[esi]			; get minute information
				inc		ah					; increment count
				
				cmp		ah,3Ah				; > 39?
				jl		doMinMS				; no, then continue
				
				;*====	Increment MS digit	====
				mov		ah,30h				; reset to  0
				inc		al					; increment MS sec digit
				
				;*====	Save the minute information	====
		doMinMS:
				mov		[esi],ax			;
		doshowex:
				ret
show_etime		endp
